"""
Django Feedme

Digest.py

Author: Derek Stegelman
"""
from django.template import loader, Context
from django.conf import settings
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives

from .models import FeedItem


def send_digest():
    """
    Send the daily digest for all users.  This sends
    both a .txt and .html version of the email.  
    :return:
    """
    text_template = loader.get_template('feedme/mail/digest.txt')
    html_template = loader.get_template('feedme/mail/digest.html')
    subject = 'Daily FeedMe Digest'

    for user in User.objects.all():
        items = FeedItem.objects.my_feed_items(user).yesterday()
        if items:
            context_dict = {'items': items}
            context = Context(context_dict)
            from_email = getattr(settings, 'FEEDME_FROM_EMAIL', 'test@test.com')
            text_content = text_template.render(context)
            html_content = html_template.render(context)
            msg = EmailMultiAlternatives(subject, text_content, from_email, [user.email])
            msg.attach_alternative(html_content, "text/html")
            msg.send()




"""
Django Feedme

Forms.py

Author: Derek Stegelman
"""

from django import forms

from .models import Feed, Category


class AddFeedForm(forms.ModelForm):
    """
    Simple add feed form
    """
    class Meta:
        model = Feed
        exclude = ('user', 'title')


class ImportFeedForm(forms.Form):
    """
    Form for importing Google reader
    """
    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        super(ImportFeedForm, self).__init__(*args, **kwargs)

        if user:
            self.fields['category'].queryset = Category.objects.filter(user=user)

    archive = forms.FileField(label='Google takeout zip file')
    category = forms.ModelChoiceField(
        label="Default category", queryset=Category.objects.filter(user=None),
        widget=forms.Select())



# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand

from feedme.models import Feed


class Command(BaseCommand):
    """
    Update all feed objects manually.
    """
    help = 'Update all feeds'

    def handle(self, *args, **options):
        for feed in Feed.objects.all():
            feed._update_feed()



"""
Django Feedme

Views.py

Author: Derek Stegelman
"""
import logging

from django.shortcuts import redirect, get_object_or_404
from django.views.generic import ListView, FormView, CreateView
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt


from infuse.auth.permissions import LoginRequiredMixin

from .models import Feed, FeedItem, Category
from .forms import AddFeedForm, ImportFeedForm
from .google_takeout import GoogleReaderTakeout
from .mixins import AjaxableResponseMixin

logger = logging.getLogger(__name__)


class FeedList(LoginRequiredMixin, ListView):
    """
    Show the feed list for the logged in user.
    """
    template_name = 'feedme/feed_list.html'
    context_object_name = 'feed_items'

    def _update_feeds(self, user):
        """
        Update the feeds.  We try to do this
        on page load in case we don't have the
        most updated feeds.
        """
        for feed in Feed.objects.filter(user=user):
            feed.update()
        return True

    def get_queryset(self):
        """
        Overwrite the query set method.  Updates the feeds
        and then checks to see if the user is passing a feed id
        or category.
        """
        # Update the feed on page load..
        self._update_feeds(self.request.user)
        items = FeedItem.objects.my_feed_items(self.request.user).un_read()

        if self.kwargs.get('category', None):
            return items.category(self.kwargs.get('category'))

        if self.kwargs.get('feed_id', None):
            return items.filter(feed__id=self.kwargs.get('feed_id'))

        return items

    def get_context_data(self, **kwargs):
        """
        Update the context data.  Add the categories and
        push the FeedForm to the template.
        """
        context = super(FeedList, self).get_context_data(**kwargs)
        context['categories'] = Category.objects.filter(user=self.request.user)
        context['add_form'] = AddFeedForm()

        return context


class ImportView(LoginRequiredMixin, FormView):
    """
    Main view for importing feeds from Google Reader.
    """
    template_name = 'feedme/takeout_form.html'
    form_class = ImportFeedForm
    success_url = 'feedme-feed-list'

    def get_context_data(self, **kwargs):
        """
        Push extra context to the template.
        :param kwargs:
        :return:
        """
        context = super(ImportView, self).get_context_data(**kwargs)
        context['add_form'] = AddFeedForm()
        context['form'] = ImportFeedForm(user=self.request.user)

        return context

    def form_valid(self, form):
        """
        Process the form.
        """
        takeout = GoogleReaderTakeout(self.request.FILES['archive'])
        for data in takeout.subscriptions():
            if not data['xmlUrl']:
                logger.info("Found feed without url. Dumping %s." % data['title'])
                continue
            if data['category']:
                category, created = Category.objects.get_or_create(name=data['category'], user=self.request.user)
            else:
                category = form.cleaned_data['category']
            Feed.objects.get_or_create(
                url=data['xmlUrl'], title=data['title'],
                user=self.request.user, last_update=None,
                category=category
            )

        return redirect(reverse(self.get_success_url()))


class AddView(LoginRequiredMixin, AjaxableResponseMixin, CreateView):
    """
    Simple class based create view
    """
    form_class = AddFeedForm
    model = Feed


def mark_all_as_read(request):
    """
    Marks all items for a user as read.
    """
    items = FeedItem.objects.my_feed_items(request.user).un_read()

    for item in items:
        item.mark_as_read()

    return redirect('feedme-feed-list')


@csrf_exempt
def mark_as_read(request):
    """
    Ajax method to mark an item as being read.
    :param request:
    :return AJAX/HTTP Response of 200:
    """
    if request.method == "POST":
        feed_item = get_object_or_404(FeedItem, pk=request.POST.get('feed_item_id'))
        feed_item.read = True
        feed_item.save()

    return HttpResponse()



"""
Django Feedme

Urls.py

Author: Derek Stegelman
"""

from django.conf.urls import patterns, url

from .views import FeedList, ImportView, AddView, mark_all_as_read, mark_as_read

urlpatterns = patterns('',
    url(r'^$', FeedList.as_view(), name="feedme-feed-list"),
    url(r'^by_category/(?P<category>[-\w]+)/$', FeedList.as_view(), name='feedme-feed-list-by-category'),
    url(r'^by_feed/(?P<feed_id>[-\w]+)/$', FeedList.as_view(), name='feedme-feed-list-by-feed'),
    url(r'^import/$', ImportView.as_view(), name='feedme-import-google-takeout'),
    url(r'^mark_all_as_read/$', mark_all_as_read, name='feedme-mark-all-as-read'),
    url(r'^ajax/mark_as_read/$', mark_as_read, name='feedme-mark-as-read-ajax'),
    url(r'^ajax/add/$', AddView.as_view(), name='feedme-add-ajax'),
)



from setuptools import setup, find_packages

setup(
    name='django-feedme',
    version='1.2.1',
    zip_safe=False,
    packages=find_packages(exclude=['demo']),
    include_package_data=True,
    url='http://github.com/dstegelman/django-feedme',
    license='MIT',
    author='Derek Stegelman',
    author_email='email@stegelman.com',
    description='Django Google Reader Replacement',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        "Framework :: Django",
    ],
    install_requires=[
        'feedparser==5.1.3,<6.0',
        'django-bootstrap-static>=2.0,<3.0',
        'hadrian>=1.1',
        'lxml',
        'django-infuse>=0.3'
    ]
)




"""
Django Feedme

Models.py

Author: Derek Stegelman
"""
import datetime

from time import mktime

from django.db import models
from django.contrib.auth.models import User
from django.conf import settings


from hadrian.utils.slugs import unique_slugify

import feedparser

from .managers import FeedItemManager


class Category(models.Model):
    """
    Category model
    """
    name = models.CharField(max_length=250, blank=True)
    slug = models.SlugField(blank=True, null=True, editable=False)
    user = models.ForeignKey(User, blank=True, null=True)

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        """
        Updated Save to slug the name.
        :param args:
        :param kwargs:
        :return:
        """
        unique_slugify(self, self.name)
        super(Category, self).save(*args, **kwargs)

    @property
    def get_unread_count(self):
        """
        Fetch the unread count for a given category.
        :return:
        """
        return FeedItem.objects.my_feed_items(self.user).category(self.slug).un_read().count()


class Feed(models.Model):
    """
    Feed Model
    """
    link = models.CharField(blank=True, max_length=450)
    url = models.CharField(blank=True, max_length=450)
    title = models.CharField(blank=True, null=True, max_length=250)
    category = models.ForeignKey(Category, blank=True, null=True)
    user = models.ForeignKey(User, blank=True, null=True)
    last_update = models.DateField(blank=True, null=True, editable=False)

    class Meta:
        unique_together = (
            ("url", "user"),
        )

    def __unicode__(self):
        return self.url

    def _get_title(self):
        """
        Fetch the title from the feed.
        :return:
        """
        parser = feedparser.parse(self.url)
        return parser.feed.title

    def save(self, *args, **kwargs):
        """
        Updated .save() method.  Fetches the title
        from the feed, and updates the record timestamp.
        :param args:
        :param kwargs:
        :return:
        """
        if not self.title:
            self.title = self._get_title()
        super(Feed, self).save(*args, **kwargs)
        if self.last_update is None:
            self.update(force=True)

    @property
    def get_unread_count(self):
        """
        Fetch the unread count for all items in this feed.
        :return int count of un-read items:
        """
        return FeedItem.objects.filter(feed=self).un_read().count()

    def _update_feed(self):
        """
        Perform an update on this feed.  This fetches the latest
        data from the feed and compares it to what we have stored currently.
        Then we go ahead and save that content and mark it as
        un-read.
        """
        # Update the last update field
        feed = feedparser.parse(self.url)
        self.last_update = datetime.date.today()
        if "link" in feed.feed:
            self.link = feed.feed.link
        else:
            self.link = ""
        self.save()
        for item in feed.entries[:10]:
            # The RSS spec doesn't require the guid field so fall back on link
            if "id" in item:
                guid = item.id
            else:
                guid = item.link

            # Search for an existing item
            try:
                FeedItem.objects.get(guid=guid)
            except FeedItem.DoesNotExist:
                # Create it.
                if "published_parsed" in item:
                    pub_date = datetime.datetime.fromtimestamp(mktime(item.published_parsed))
                elif "updated_parsed" in item:
                    pub_date = datetime.datetime.fromtimestamp(mktime(item.updated_parsed))
                else:
                    pub_date = datetime.datetime.now()

                feed_item = FeedItem(title=item.title, link=item.link, content=item.description,
                                     guid=guid, pub_date=pub_date, feed=self)
                feed_item.save()

    def _update_processor(self):
        """
        Kick off the prrocessing of the feeds.  Either update with celery
        or in real time if we aren't using Celery.
        :return:
        """
        if getattr(settings, 'FEED_UPDATE_CELERY', False):
            from .tasks import update_feed
            update_feed.delay(self)
            return True
        self._update_feed()
        return True

    def update(self, force=False):
        """
        If we aren't forcing it
        and its not the same day, go ahead
        and update the feeds.
        """

        if force or not force and self.last_update < datetime.date.today():
            self._update_processor()
            return True

    @models.permalink
    def get_absolute_url(self):
        """
        Feed permalink
        :return:
        """
        return ('feedme-feed-list-by-feed', (), {'feed_id': self.id})


class FeedItem(models.Model):
    """
    FeedItem Model
    """
    title = models.CharField(max_length=350, blank=True)
    link = models.URLField(blank=True)
    content = models.TextField(blank=True)
    feed = models.ForeignKey(Feed, blank=True, null=True)
    read = models.BooleanField(default=False)
    guid = models.CharField(max_length=255)
    date_fetched = models.DateField(auto_created=True, auto_now_add=True, editable=True)
    pub_date = models.DateTimeField()

    objects = FeedItemManager()

    class Meta:
        ordering = ['id']

    def __unicode__(self):
        return self.title

    def mark_as_read(self):
        """
        Mark an item as read.
        """
        self.read = True
        self.save()




# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand

from feedme import digest


class Command(BaseCommand):
    """
    Management command to send the daily digest.  Typically
    called by Celery.
    """
    help = 'Send Digest'

    def handle(self, *args, **options):
        digest.send_digest()
